/* * WeSocket HTML+JavaScript
 * Ewald Kendziorra sr, 2015, http://kendziorra.nl
 * License: Free
 * Publishing: Only with this header
 * Editing: Free
 * Guarantee: No
 * Using: At your own risk
 * Support: No
 **************************************************************************************************
 * AndroidInHtml: Up to 21 (Lollipop) Android sensors, GPS, ... available in HTML5+Javascript
 * Save HTMLs and scripts on your Android SD-card and open it like this
 *   file:///storage/extSdCard/AndroidInHtml/AndroidInHtml.html
 * in the AndroidInHtml APP
 **************************************************************************************************
 * SolPosSPA is based on Solar Position Algorithm (SPA):                                                                                      
 * Valid from the year -2000 to 6000, with an uncertainty of +/- 0.003 degrees.
 * Converted from C code to C# and JavaScipt by Ewald Kendziorra sr (read header in SolPosSPA.js)
 **************************************************************************************************
 * Combine this code with my ESP8266(+Arduino) Publish/Subscribe project
 **************************************************************************************************/

// http://developer.android.com/guide/topics/sensors/sensors_overview.html
// http://developer.android.com/guide/topics/sensors/sensors_motion.html
// http://developer.android.com/guide/topics/sensors/sensors_position.html
// http://developer.android.com/guide/topics/sensors/sensors_environment.html
// http://developer.android.com/reference/android/hardware/SensorEvent.html#values
// http://developer.android.com/reference/android/location/Location.html

/*******************************************************************
 ********************* SENSORS and LOCATION ************************
 * sensor list Android API level 17 (Kitkat v4.2) -> 22 (Lollipop) *
 *******************************************************************/
  var ACCELEROMETER = 1, MAGNETIC_FIELD = 2, ORIENTATION = 3, GYROSCOPE = 4, LIGHT = 5, PRESSURE = 6,
      TEMPERATURE = 7, PROXIMITY = 8, GRAVITY = 9, LINEAR_ACCELERATION = 10, ROTATION_VECTOR = 11,
      RELATIVE_HUMIDITY = 12, AMBIENT_TEMPERATURE = 13, MAGNETIC_FIELD_UNCALIBRATED = 14,
      GAME_ROTATION_VECTOR = 15, GYROSCOPE_UNCALIBRATED = 16, SIGNIFICANT_MOTION = 17,
      STEP_DETECTOR = 18, STEP_COUNTER = 19, GEOMAGNETIC_ROTATION_VECTOR = 20, HEART_RATE = 21;
  var COMPASS = 50, LOCATION = 60;  // extra information
  
  var toastMsg = "";     // message holder
  var toastTime = 5;  // display Toast 5 sec.

  function sensors(sensorType) {  // Called from Android
    switch (sensorType) {
      case ACCELEROMETER: $("#chbAccelerometer").prop("disabled", false); break;
      case MAGNETIC_FIELD: $("#chbMagneticField").prop("disabled", false); break;
      case ORIENTATION: $("#chbOrientation").prop("disabled", false); break;
      case GYROSCOPE: $("#chbGyroscope").prop("disabled", false); break;
      case LIGHT: $("#chbLight").prop("disabled", false); break;
      case PRESSURE: $("#chbPressure").prop("disabled", false); break;
      case TEMPERATURE: $("#chbTemperature").prop("disabled", false); break;
      case PROXIMITY: $("#chbProximity").prop("disabled", false); break;
      case GRAVITY: $("#chbGravity").prop("disabled", false); break;
      case LINEAR_ACCELERATION: $("#chbLinearAcceleration").prop("disabled", false); break;
      case ROTATION_VECTOR: $("#chbRotationVector").prop("disabled", false); break;
      case RELATIVE_HUMIDITY: $("#chbRelativeHumidity").prop("disabled", false); break;
      case AMBIENT_TEMPERATURE: $("#chbAmbientTemperature").prop("disabled", false); break;
      case MAGNETIC_FIELD_UNCALIBRATED: $("#chbMagneticFieldUncalibrated").prop("disabled", false); break;
      case GAME_ROTATION_VECTOR: $("#chbGameRotationVector").prop("disabled", false); break;
      case GYROSCOPE_UNCALIBRATED: $("#chbGyroscopeUncalibrated").prop("disabled", false); break;
      case SIGNIFICANT_MOTION: $("#chbSignificantMotion").prop("disabled", false); break;
      case STEP_DETECTOR: $("#chbStepDetector").prop("disabled", false); break;
      case STEP_COUNTER: $("#chbStepCounter").prop("disabled", false); break;
      case GEOMAGNETIC_ROTATION_VECTOR: $("#chbGeomagneticRotationVector").prop("disabled", false); break;
      case HEART_RATE: $("#chbHeartRate").prop("disabled", false); break;
    }
  }
  
  $(":checkbox").click(function(){
    Android.setSensorFunc(this.value, this.checked);
  }); 
  
  function setCompass() {
    if ($("#chbAccelerometer").prop("checked") && $("#chbMagneticField").prop("checked")) 
      $("#chbCompass").prop("disabled", false);
    else {
      Android.setSensorFunc(COMPASS, false);
      $("#chbCompass").prop("checked", false);
      $("#chbCompass").prop("disabled", true);
    }
  }
 
  function sensorData(sensorType, data) {  // Called from Android
    switch (sensorType) {
      case ACCELEROMETER: $("#Accelerometer").text(data.formatArray(3, 5, 4, " ")); setCompass(); break;
      case MAGNETIC_FIELD: $("#MagneticField").text(data.formatArray(3, 5, 4, " ")); setCompass(); break;
      case ORIENTATION: $("#Orientation ").text(data.formatArray(3, 5, 4, " ")); break;
      case GYROSCOPE: $("#Gyroscope").text(data.formatArray(3, 5, 4, " ")); break;
      case LIGHT: $("#Light").text(data.formatArray(1, 5, 4, " ")); break;
      case PRESSURE: $("#Pressure").text(data.formatArray(1, 5, 4, " ")); break;
      case TEMPERATURE: $("#Temperature").text(data.formatArray(1, 5, 4, " ")); break;
      case PROXIMITY: $("#Proximity").text(data.formatArray(1, 5, 4, " ")); break;
      case GRAVITY: $("#Gravity").text(data.formatArray(3, 5, 4, " ")); break;
      case LINEAR_ACCELERATION: $("#LinearAcceleration ").text(data.formatArray(3, 5, 4, " ")); break;
      case ROTATION_VECTOR: $("#RotationVector").text(data.formatArray(3, 5, 4, " ")); break;
      case RELATIVE_HUMIDITY: $("#RelativeHumidity").text(data.formatArray(1, 5, 4, " ")); break;
      case AMBIENT_TEMPERATURE: $("#AmbientTemperature").text(data.formatArray(1, 5, 4, " ")); break;
      case MAGNETIC_FIELD_UNCALIBRATED: $("#MagneticFieldUncalibrated").text(data.formatArray(6, 5, 4, " ")); break;
      case GAME_ROTATION_VECTOR: $("#GameRotationVector").text(data.formatArray(3, 5, 4, " ")); break;
      case GYROSCOPE_UNCALIBRATED: $("#GyroscopeUncalibrated").text(data.formatArray(6, 5, 4, " ")); break;
      case SIGNIFICANT_MOTION:  // this is a single trigger action  
        $("#SignificantMotion").text(Date(data));
        Android.setSensorFunc(17, false);  // motion off
        $("#chbSignificantMotion").prop('checked', false);  // uncheck
        Android.Alert(100); // 100 msec.
        Android.showToast("Motion detected!");
        break;
      case STEP_DETECTOR: 
        $("#StepDetector").text(Date(data));
        Android.Alert(100); // 100 msec.
        Android.showToast("Step Detected");
        break;
      case STEP_COUNTER: $("#StepCounter").text(data.formatArray(1, 5, 4, " ")); break;
      case GEOMAGNETIC_ROTATION_VECTOR: $("#GeomagneticRotationVector").text(data.formatArray(3, 5, 4, " ")); break;
      case HEART_RATE: $("#HeartRate").text(data.formatArray(1, 5, 4, " ")); break;
      case COMPASS:
        $('#Azimuth').text(Math.radToDeg(data[0]).formatN(4, 4, ' '));
        $('#Pitch').text(Math.radToDeg(data[1]).formatN(4, 4, ' '));
        $('#Roll').text(Math.radToDeg(data[2]).formatN(4, 4, ' ')); break;
      case LOCATION:
        $('#Latitude').text(data[0].formatN(4, 4, ' '));
        $('#Longitude').text(data[1].formatN(4, 4, ' '));
        $('#Altitude').text(data[2].formatN(4, 4, ' '));
        $('#Bearing').text(data[3].formatN(4, 4, ' '));
        $('#Speed').text(data[4].formatN(4, 2, ' '));
        $('#Accuracy').text(data[5].formatN(4, 2, ' ')); break;
    }
  }

/****************************************************
 ************************ SUN ***********************
 ****************************************************/
  const JUN = 1, DEC = 3;  // Solstice offset: longest/shortest day of the year
  var dat, day, month, year;

  if (typeof(Storage) != "undefined") {  // Retrieve saved Lat/Lon from HTML5 storage
    var lat = localStorage.getItem("latitude");
    var lon = localStorage.getItem("longitude");
    if (lat == undefined) lat = "52.0000";
    if (lon == undefined) lon = "5.0000"; 
    $("#lat").val(lat);
    $("#lon").val(lon);
  } 

  $('#btnDeltaT').click(function() {  // calc ?T 2005-2050, based on Meeus Astronmical Algorithms
    dat = new Date();
    year = dat.getFullYear();
    month = dat.getMonth();
    if (month > 3) year += 0.25;
    if (month > 6) year += 0.25;
    if (month > 9) year += 0.25;
    $("#deltaT").val((62.92 + 0.32217 * (year - 2000) + 0.005589 * Math.pow(year - 2000, 2) - 1.31).formatN( 3, 2, " "));
  });
  
  $('#btnSun').click(function() {  // start new calculation
    if (typeof(Storage) != "undefined") {  // Store last used Lat/Lon to HTML5 storage
      localStorage.setItem("latitude", $("#lat").val());
      localStorage.setItem("longitude", $("#lon").val());
    }
    // init SunPos SPA
    dat = new Date();
    year = dat.getFullYear();
    month = dat.getMonth() + 1;  // jan = 0
    day = dat.getDate();
    var timezone = -dat.getTimezoneOffset() / 60;
    var latitude = $("#lat").val().parseF();  // from location?
    // Solstice Jun/Dec: calc longest and shortest day of the year
    var junDate = calcEquiSol(JUN, year);  // get jun day
    var decDate = calcEquiSol(DEC, year);  // get dec day
    junDate.addMinutes(latitude >= 0 ? dat.DST() : 0);  // jun DST?
    decDate.addMinutes(latitude >= 0 ? 0 : dat.DST());  // dec DST?
    // show Solstice 
    $("#Jun").text(junDate.getDate());
    $("#Dec").text(decDate.getDate());
    // init SPA
    spaIn.year = year;
    spaIn.month = month;
    spaIn.day = day;
    spaIn.hour = dat.getHours();
    spaIn.minute = dat.getMinutes();
    spaIn.second = dat.getSeconds();
    spaIn.latitude = latitude;
    spaIn.longitude = $("#lon").val().parseF();  // can also used from location?
    spaIn.timezone = timezone;
    spaIn.elevation = $("#ele").val().parseF();  // can also used from location?
    spaIn.pressure = $("#prs").val().parseF();  // can also used from KitKat+ sensor?
    spaIn.temperature = $("#tmp").val().parseF();  // can also used from KitKat+ sensor?
    spaIn.delta_t = $("#deltaT").val().parseF();
    spaIn.slope = 0;
    spaIn.azm_rotation = 180;
    spaIn.atmos_refract = 0.5667;
    spaIn.function = SPA_ZA_RTS;  // see SunPosSPA.js "const int <calculation type>"
    if (spa_calculate() == 0) {  // calc SUN with NREL's SPA
      // show PSA 
      $("#Time").text(dat);
      $("#SunRise").text(spaOut.sunrise.timeF(timezone));
      $("#Transit").text(spaOut.suntransit.timeF(timezone));
      $("#SunSet").text(spaOut.sunset.timeF(timezone));
      $("#SunAzimuth").text(spaOut.azimuth180.formatN(4, 5, " "));
      $("#SunAltitude").text((90 - spaOut.zenith).formatN(4, 5, " "));
    }
    else alert("Error: check SPA input!");
  });

/****************************************************
 ********************* diverse **********************
 ****************************************************/
  $('#btnVibrate').click(function() {  // Android vibrator - to Android
    Android.Vibrate(500); // 1/2 sec.
  });
  
  $('#btnAlert').click(function() {  // Android alert signal (ToneGenerator) - to Android
    Android.Alert(500); // 1/2 sec.
  });
  
  var toastMsg = "Hello from JavaScript to Android!!!";
  $('#btnToast').click(function() {  // Android toast / alternative to JavaScript allert(msg) - to Android
    Android.showToast(toastMsg);
  });
  
  $('#btnSms').click(function() {  // Android send SMS - to Android
    Android.SendSMS($("#phoneNumber").val(), $("#message").val());
  });
  
  $('#btnMail').click(function() {  // Android send e-mail - to Android
    Android.SendMail($("#user").val(), $("#password").val(), $("#toArr").val().replace(/\s/g, "").split(";"), $("#subject").val(), $("#body").val(), $("#attachment").val());
    // result: Android Toast message
  });

  /******* Camera ****/
  $('#btnPhoto').click(function() {  // Android take a picture - to Android
    // set parameters
    Android.paramPhoto($("#chbFront").prop("checked"), $("#chbPortrait").prop("checked"), $("#chbFlash").prop("checked"), $("#size").val());
    // take a picture
    Android.takePhoto($("#folder").val());  // filenaam is auto generated as time in <microseconds>.jpg
    // result: Android Take a picture or ERROR in callback function photoSaved(isSaved)
  });
          
  var photoFile = "";  // save filepath
  function photoSaved(pathFile) { // Called from Android - callback of TakePhoto(): path or ERROR
    photoFile = pathFile;  // save filepath
    $('#photoSaved').text(photoFile );  // show filepath
  }

  /******* (OTG) USB ****/
  $('#btnSerial').click(function() {  // Android send Serial message to ESP/Arduino - to Android
    Android.toUsb($("#usbSend").val());
  });
  
  var inpBuf = ""; 
  function usbReceived(msg) {  // Called from Android - receive Serial message from ESP/Arduino
    inpBuf += msg;
    $('#usbReceive').text(inpBuf);  // show message
  }
  
  $('#btnCls').click(function() {  // Clear Serial message 
    inpBuf = "";
    $('#usbReceive').text(inpBuf); 
  });
  
/****************************************************
 ******************** HELPERS ***********************
 ****************************************************/
  /***** radians and degrees conversion *****/
  Math.radToDeg = function(rad)
  { return rad*(180/Math.PI); }
  Math.degToRad = function(deg)
  { return deg * (Math.PI/180); }
  /***** DateTime *****/
  Date.prototype.addHours = function(h)
  { return this.setTime(this.getTime() + (h * 3600000)); }
  Date.prototype.addMinutes = function(h)
  { return this.setTime(this.getTime() + (h * 60000)); } 
  Number.prototype.timeF = function(h)
  { var d = new Date(new Date(year, month, day, 0, 0, 0, 0).addHours(this + h));
    return d.getHours().toString().PadLeft(2, "0") + ":" + d.getMinutes().toString().PadLeft(2, "0") + ":" + d.getSeconds().toString().PadLeft(2, "0"); }
  /*******************************************
   ***** Array Numbers and String Format *****/
  // n=1->n items l=length Left, r=length right, f=fill char
  Array.prototype.formatArray = function(n, l, r, f)
  { var str = ""; for (var i = 0; i < n; i++) str += parseFloat(this[i]).formatN(l, r, f); return str; }
  /***** Numbers and String Format *****/
  // l=length Left, r=length right, f=fill char
  Date.prototype.DST = function()  // get DST minutes
  { return (Math.max(new Date(this.getFullYear(), 0, 1).getTimezoneOffset(), new Date(this.getFullYear(), 6, 1).getTimezoneOffset()) - this.getTimezoneOffset()); }
  Number.prototype.formatN = function(l, r, f)  // format Number to PadLeft String
  { return this.toFixed(r).toString().PadLeft(l + (r == 0 ? 0 : ++r), f); }
  String.prototype.PadLeft = function(l, f)  // format String
  { var s = this; if (f == " ") f = '\xa0'; if (s.length < l) for (i = s.length; i < l; i++) s = f + s; return s; }
  String.prototype.parseF = function()  // format Number to String
  { return parseFloat(this.replace(",",".")); }
  /*********************************************/
  
  Android.htmlIsLoaded();  //  to Android: HTML page and script are loaded 
