﻿/******************************************************************
 * Equinox and Solstice
 * Based on: Astronomical Algorithms, by Jean Meeus, 2nd ed, 1998
 * usage:
 *  DateTime Res = EquSol(month, year);
 *  // month: 0=Mar, 1=Jun, 2=Sep, 3=Dec
********************************************************************/
var A = new Array(485, 203, 199, 182, 156, 136, 77, 74, 70, 58, 52, 50, 45, 44, 29, 18, 17, 16, 14, 12, 12, 12, 9, 8);
var B = new Array(324.96, 337.23, 342.08, 27.85, 73.14, 171.52, 222.54, 296.72, 243.58, 119.81, 297.17,
    21.02, 247.54, 325.15, 60.93, 155.12, 288.79, 198.04, 199.76, 95.39, 287.11, 320.81, 227.73, 15.45);
var C = new Array(1934.136, 32964.467, 20.186, 445267.112, 45036.886, 22518.443, 65928.934,
    3034.906, 9037.513, 33718.147, 150.678, 2281.226, 29929.562, 31555.956, 4443.417, 67555.328,
    4562.452, 62894.029, 31436.921, 14577.848, 31931.756, 34777.259, 1222.114, 16859.074);

function calcEquiSol(month, year)
{
  var JDE0 = calcInitial(month, year);
  var T = (JDE0 - 2451545.0) / 36525;
  var W = 35999.373 * T - 2.47;
  var dL = 1 + 0.0334 * Math.cos(W * Math.PI / 180) + 0.0007 * Math.cos(2 * W * Math.PI / 180);
  var S = periodic24(T);
  var JDE = JDE0 + ((0.00001 * S) / dL);
  var TDT = fromJDtoUTC(JDE);
  return fromTDTtoUTC(TDT);
}

function calcInitial(i, year)
{
  var JDE0 = 0, Y = (year-2000) / 1000;
  switch(i) {
    case 0: JDE0 = 2451623.80984 + 365242.37404 * Y + 0.05169 * Math.pow(Y, 2) - 0.00411 * Math.pow(Y, 3) - 0.00057 * Math.pow(Y, 4); break;
    case 1: JDE0 = 2451716.56767 + 365241.62603 * Y + 0.00325 * Math.pow(Y, 2) + 0.00888 * Math.pow(Y, 3) - 0.00030 * Math.pow(Y, 4); break;
    case 2: JDE0 = 2451810.21715 + 365242.01767 * Y - 0.11575 * Math.pow(Y, 2) + 0.00337 * Math.pow(Y, 3) + 0.00078 * Math.pow(Y, 4); break;
    case 3: JDE0 = 2451900.05952 + 365242.74049 * Y - 0.06223 * Math.pow(Y, 2) - 0.00823 * Math.pow(Y, 3) + 0.00032 * Math.pow(Y, 4); break;
  }
  return JDE0;
}

function periodic24(T)
{
  var S = 0;
  for (var i = 0; i < 24; i++) S +=  A[i] * Math.cos((B[i] + (C[i] * T)) * Math.PI / 180);
  return S;
} 

function fromTDTtoUTC(tobj)
{
  var Year = tobj.getUTCFullYear();
  var deltaT = 62.92 + 0.32217 * (Year - 2000) + 0.005589 * Math.pow(Year - 2000, 2) - 1.31;
  return(new Date(tobj.getTime() - (deltaT * 1000)));
}

function fromJDtoUTC(JD)
{
  var A, alpha;
  var Z =  Math.floor( JD + 0.5);
  var F = (JD + 0.5) - Z;
  if (Z < 2299161) A = Z;
  else
  {
    alpha =  Math.floor( (Z-1867216.25) / 36524.25);
    A = Z + 1 + alpha -  Math.floor( alpha / 4);
  }
  var B = A + 1524;
  var C =  Math.floor( (B-122.1) / 365.25);
  var D =  Math.floor( 365.25 * C);
  var E =  Math.floor( (B-D) / 30.6001);
  var DT = B - D -  Math.floor(30.6001 * E) + F;
  var Mon = E - (E < 13.5?1:13);
  var Yr  = C - (Mon>2.5?4716:4715);   
  var Day =  Math.floor( DT);
  var H = 24 * (DT - Day);
  var Hr =  Math.floor( H);
  var M = 60 * (H - Hr);
  var Min =  Math.floor( M);
  var Sec =  Math.floor( 60 * (M-Min));
  var theDate = new Date(0);
  theDate.setUTCFullYear(Yr, Mon-1, Day);
  theDate.setUTCHours(Hr, Min, Sec);
  return(theDate);
}

